
# q-expansion of newform 5782.2.a.e, downloaded from the LMFDB on 01 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5782
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [-2], [2], [0], [-1], [3], [-7], [-4], [4], [4], [4], [-7], [11], [9], [-10], [0], [1], [2], [4], [9], [14], [11], [13], [-18], [-2], [9], [10], [6], [-14], [8], [-8], [12], [9], [10], [-5], [2], [-18], [22], [-12], [11], [-7], [4], [6], [25], [-12], [-16], [-5], [-21], [25], [13], [0], [8], [25], [2], [-9], [3], [19], [1], [18], [-1], [19], [8], [-28], [5], [-18], [16], [-12], [22], [-4], [-1], [-22], [27], [32], [10], [14], [25], [-4], [14], [6], [14], [3], [-11], [12], [-13], [-35], [9], [-25], [-10], [18], [6], [-1], [32], [-29], [10], [-18], [-30], [-42], [-10], [14], [-43], [22], [0], [-4], [24], [-20], [19], [15], [-34], [27], [-10], [21], [5], [-31], [16], [11], [-18], [-46], [-20], [-18], [-28], [-50], [-18], [-12], [37], [29], [27], [52], [14], [23], [22], [-25], [-48], [-12], [-10], [22], [0], [6], [12], [-23], [-6], [-9], [25], [-14], [-12], [-4], [-30], [-54], [-38], [-20], [-14], [20], [-8], [-34], [6], [-10], [17], [38], [40], [40], [2], [10], [31], [-30], [-6], [-12], [-18], [-24], [52], [-26], [-33], [-46], [13], [-3], [36], [13], [51], [-22], [-37], [40], [6], [32], [12], [-31], [-30], [3], [6], [-42], [27], [4], [-18], [38], [-35], [-1], [-54], [-12], [-54], [-10], [-37], [66], [-46], [6], [56], [-24], [-48], [-38], [61], [-30], [49], [34], [-25], [30], [12], [-21], [-42], [-23], [8], [49], [-3], [-60], [-30], [-39], [-42], [-4], [44], [3], [-55], [66], [8], [6], [18], [-59], [-15], [-44], [0], [-36], [0], [-64], [26], [-12], [-11], [60], [-35], [1], [-36], [15], [35], [-8], [-13], [30], [-57], [60], [-66], [6], [-5], [68], [-18], [-4], [76], [-70], [7], [-64], [48], [-4], [44], [50], [-19], [0], [0], [-18], [-6], [-62], [-12], [28], [-5], [57], [28], [-6], [-80], [-40], [-54], [-67], [46], [48], [61], [-60], [78], [36], [68], [22], [30], [55], [6], [66], [75], [10], [-21], [27], [58], [15], [-84], [6], [23], [2], [31], [80], [56], [30], [-61], [60], [38], [59], [24], [-25], [17], [45], [-50], [-44], [33], [-66], [-3], [62], [36], [-24], [20], [46], [-38], [-29], [-40], [76], [-86], [-15], [78], [-70], [34], [54], [2], [-44], [-24], [-30], [-8], [46], [-14], [17], [48], [-69], [-83], [-31], [74], [28], [-18], [-20], [-37], [-72], [64], [28], [66], [40], [-54], [94], [-51], [-59], [-7], [13], [13], [18], [16], [27], [-65], [-30], [30], [-55], [-38], [6], [10], [-12], [24], [-34], [-62], [6], [-58], [14], [72], [-67], [-22], [-94], [-72], [-67], [-72], [4], [56], [10], [51], [-79], [37], [-52], [-22], [0], [-18], [38], [-74], [-70], [-42], [53], [53], [74], [-56], [-62], [-43], [-49], [-60], [25], [-58], [96], [74], [-41], [-28], [70], [102], [42], [79], [94], [-10], [72], [-86], [59], [42], [20], [31], [-10], [-32]]
